unit UnitDefiniceVyjimek;

interface

uses SysUtils;

type
  ODTCisloExceptionCode = (DIV_BY_ZERO,
                           INVALID_LOG_ARGUMENT,
                           INVALID_SQRT_ARGUMENT,
                           LOSS_OF_PRECISION);
  VarsExceptionCode = (VAR_ALREADY_EXISTS, VAR_NOT_FOUND);
  MiscExceptionCode = (EXECUTION_TERMINATED, UNSPECIFIED_ERROR);
  RuntimeExceptionCode = (WRONG_PARAM_FORMAT,
                          PROC_LAUNCH_ABORTED,
                          CALL_STACK_OVERFLOW,
                          PARAM_STACK_OVERFLOW,
                          VARIABLE_STACK_OVERFLOW,
                          DUPLICITE_ENTRY);
  ParseExceptionCode = (
      SEMICOLON_MISSING,
      VPRED_O_MISSING,
      OTOC_O_MISSING,
      SMAZ_EXCL_MISSING,
      OPAKUJ_KRAT_MISSING,
      OPAKUJ_COLON_MISSING,
      OPAKUJ_KONEC_MISSING,
      POKUD_KONEC_EXPECTED,
      POKUD_COLON_EXPECTED,
      POKUD_POKUD_EXPECTED,
      POKUD_INVALID_CONSTRUCTION,
      NUMBER_EXPECTED,
      OPENING_BRACKET_EXPECTED,
      CLOSING_BRACKET_EXPECTED,
      TLOUSTKA_BUD_EXPECTED,
      TLOUSTKA_NOT_INTEGER,
      UNKNOWN_COLOR,
      AT_JE_EXPECTED,
      AT_VARNAME_EXPECTED,
      AT_EQUALS_EXPECTED,
      VAR_EXPECTED,
      POCITEJ_OD_EXPECTED,
      POCITEJ_DO_EXPECTED,
      POCITEJ_PO_EXPECTED,
      POCITEJ_COLON_EXPECTED,
      POCITEJ_KONEC_MISSING,
      POCITEJ_INTEGER_EXPECTED,
      POCITEJ_ZERO_STEP,
      UNKNOWN_TYPE,
      UNKNOWN_COMMAND,
      UNKNOWN_FUNCTION,
      OTOC_UNKNOWN_DIRECTION,
      UNEXPECTED_FILE_END,
      ZELVICKU_UNKNOWN_ACTION,
      COMPARISON_SIGN_EXPECTED,
      DOKUD_KONEC_EXPECTED,
      DOKUD_COLON_EXPECTED,
      PROVADEJ_COLON_EXPECTED,
      PROVADEJ_NEZ_EXPECTED,
      NACTI_VARNAME_EXPECTED,
      PRECTI_QUOTES_EXPECTED,
      NAPIS_QUOTES_EXPECTED
  );


  ParseException = class (Exception)
    private
      kodChyby: ParseExceptionCode;
      function  getErrType: ParseExceptionCode;
    public
      constructor Create (kod: ParseExceptionCode);
      property    errType: ParseExceptionCode read getErrType;
  end;


  RuntimeException = class (Exception)
    private
      kodChyby: RuntimeExceptionCode;
      function  getErrType: RuntimeExceptionCode;
    public
      constructor Create (kod: RuntimeExceptionCode);
      property    errType: RuntimeExceptionCode read getErrType;
  end;


  ODTCisloException = class (Exception)
    private
      kodChyby: ODTCisloExceptionCode;
      function  getErrType: ODTCisloExceptionCode;
    public
      constructor Create (kod: ODTCisloExceptionCode);
      property    errType: ODTCisloExceptionCode read getErrType;
  end;


  MiscException = class (Exception)
    private
      kodChyby: MiscExceptionCode;
      function  getErrType: MiscExceptionCode;
    public
      constructor Create (kod: MiscExceptionCode);
      property    errType: MiscExceptionCode read getErrType;
  end;


  VarsException = class (Exception)
    private
      kodChyby: VarsExceptionCode;
      function  getErrType: VarsExceptionCode;
    public
      constructor Create (kod: VarsExceptionCode);
    published
      property errType: VarsExceptionCode read getErrType;
  end;

implementation


constructor ODTCisloException.Create (kod: ODTCisloExceptionCode);
begin
    inherited Create ('Chyba ve vypoctu.');
    kodChyby := kod;
end;


constructor VarsException.Create (kod: VarsExceptionCode);
begin
    inherited Create ('Chyba pri praci s promennymi.');
    kodChyby := kod;
end;


constructor ParseException.Create (kod: ParseExceptionCode);
begin
    inherited Create ('Chyba pri analyze zdrojoveho kodu.');
    kodChyby := kod;
end;


constructor MiscException.Create (kod: MiscExceptionCode);
begin
    inherited Create ('Nespecifikovana chyba.');
    kodChyby := kod;
end;


constructor RuntimeException.Create (kod: RuntimeExceptionCode);
begin
    inherited Create ('Chyba pri behu programu.');
    kodChyby := kod;
end;


function VarsException.getErrType: VarsExceptionCode;
begin
    getErrType := kodChyby;
end;


function ODTCisloException.getErrType: ODTCisloExceptionCode;
begin
    getErrType := kodChyby;
end;


function ParseException.getErrType: ParseExceptionCode;
begin
    getErrType := kodChyby;
end;


function MiscException.getErrType: MiscExceptionCode;
begin
    getErrType := kodChyby;
end;


function RuntimeException.getErrType: RuntimeExceptionCode;
begin
    getErrType := kodChyby;
end;


end.
