unit UnitObecnyDatovyTyp;

interface

uses UnitDefiniceVyjimek;

const PI = 3.141592654;
const EULER = 2.718281828;

type

  ODTTyp = (CISLO_CELE, CISLO_DESETINNE, LOGICKA_HODNOTA);

  ODT = class
    protected
      typ: ODTTyp;   {vkladame-li obecny datovy typ do TStringList,
                      ztrati se informace o typu (ten je TObject),
                      nutno si zapamatovat}
    public
      procedure Read (const x: ODT);
      function getType: ODTTyp;
  end;


  ODTLogickaHodnota = class (ODT)
    private
      lValue: boolean;
    public
      property value: boolean read lValue write lValue;
      constructor Create (v: boolean);
      function negated: ODTLogickaHodnota;
      procedure conjunction (const operand2: ODTLogickaHodnota);
      procedure disjunction (const operand2: ODTLogickaHodnota);
      procedure Read (const x: ODTLogickaHodnota);
  end;


  ODTCislo = class (ODT)
    private
      cele:         longint;
      desetinne:    real;
    public
      constructor Create; overload;
      constructor Create (decimal: boolean); overload;
      constructor Create (x: integer); overload;
      constructor Create (x: real); overload;
      constructor Create (x: ODTCislo); overload;
      procedure Read (x: ODTCislo); overload;
      procedure Read (x: integer); overload;
      procedure Add (const x: ODTCislo); overload;
      procedure Add (x: integer); overload;
      procedure Subtr (const x: ODTCislo);
      procedure Multip (const x: ODTCislo); overload;
      procedure Multip (x: integer); overload;
      procedure DivideBy (const x: ODTCislo);
      procedure Negate;
      function  Truncated: ODTCislo;
      function  Rounded: ODTCislo;
      function  getReal: real;
      function  getInteger: longint;
      function  fSin: ODTCislo;
      function  fCos: ODTCislo;
      function  fLn: ODTCislo;
      function  fLog: ODTCislo;
      function  fSqrt: ODTCislo;
      function  jeCele: boolean;
  end;

implementation


procedure ODT.Read(const x: ODT);
begin
end;


function ODT.getType: ODTTyp;
begin
    getType := typ;
end;


constructor ODTLogickaHodnota.Create (v: boolean);
begin
    typ := LOGICKA_HODNOTA;
    inherited Create;
    lValue := v;
end;


function ODTLogickaHodnota.negated;
begin
    negated := ODTLogickaHodnota.Create (not (lValue));
end;


procedure ODTLogickaHodnota.conjunction (const operand2: ODTLogickaHodnota);
begin
    if not operand2.value then lValue := false;
end;


procedure ODTLogickaHodnota.disjunction (const operand2: ODTLogickaHodnota);
begin
    if operand2.value then lValue := true;
end;


procedure ODTLogickaHodnota.Read (const x: ODTLogickaHodnota);
begin
    lValue := x.lValue;
end;


function ODTCislo.jeCele: boolean;
begin
    jeCele := (typ = CISLO_CELE);
end;


constructor ODTCislo.Create;
begin
    TObject (Self).Create;
    typ := CISLO_CELE;
    cele := 0;
    desetinne := 0;
end;


constructor ODTCislo.Create (decimal: boolean);
begin
    inherited Create;
    if decimal then typ := CISLO_DESETINNE
               else typ := CISLO_CELE;
    cele := 0;
    desetinne := 0;
end;


constructor ODTCislo.Create (x: integer);
begin
    TObject (Self).Create;
    typ := CISLO_CELE;
    cele := x;
end;


constructor ODTCislo.Create (x: real);
begin
    TObject (Self).Create;
    typ := CISLO_DESETINNE;
    desetinne := x;
end;


constructor ODTCislo.Create (x: ODTCislo);
begin
    TObject (Self).Create;
    typ := x.typ;
    desetinne := x.desetinne;
    cele := x.cele;
end;


procedure ODTCislo.Read (x: ODTCislo);
begin
    if (typ = CISLO_CELE) and (x.typ = CISLO_DESETINNE) then
        raise ODTCisloException.Create (LOSS_OF_PRECISION);
    if typ = x.typ then begin
        desetinne := x.desetinne;
        cele := x.cele;
    end else begin
        desetinne := x.cele;
    end;
end;


procedure ODTCislo.Read (x: integer);
begin
    typ := CISLO_CELE;
    cele := x;
end;


procedure ODTCislo.Add (const x: ODTCislo);
begin
    if (typ = CISLO_CELE) and (x.typ = CISLO_CELE) then begin
        cele := cele + x.cele;
    end else if typ = CISLO_CELE then begin
        typ := CISLO_DESETINNE;
        desetinne := cele + x.desetinne;
    end else if x.typ = CISLO_CELE then begin
        desetinne := desetinne + x.cele;
    end else
        desetinne := desetinne + x.desetinne;
end;


procedure ODTCislo.Add (x: integer);
begin
    if typ = CISLO_DESETINNE then desetinne := desetinne + x
                             else cele := cele + x;
end;


procedure ODTCislo.Negate;
begin
    cele := -cele;
    desetinne := -desetinne;
end;


procedure ODTCislo.Subtr (const x: ODTCislo);
begin
    if (typ = CISLO_CELE) and (x.typ = CISLO_CELE) then begin
        cele := cele - x.cele;
    end else if typ = CISLO_CELE then begin
        typ := CISLO_DESETINNE;
        desetinne := cele - x.desetinne;
    end else if x.typ = CISLO_CELE then begin
        desetinne := desetinne - x.cele;
    end else
        desetinne := desetinne - x.desetinne;
end;


procedure ODTCislo.Multip (const x: ODTCislo);
begin
    if (typ = CISLO_CELE) and (x.typ = CISLO_CELE) then begin
        cele := cele * x.cele;
    end else if typ = CISLO_CELE then begin
        typ := CISLO_DESETINNE;
        desetinne := cele * x.desetinne;
    end else if x.typ = CISLO_CELE then begin
        desetinne := desetinne * x.cele;
    end else
        desetinne := desetinne * x.desetinne;
end;


procedure ODTCislo.Multip (x: integer);
begin
    if typ = CISLO_DESETINNE then desetinne := desetinne * x
                             else cele := cele * x;
end;


function  ODTCislo.Truncated: ODTCislo;
begin
    Truncated := ODTCislo.Create;
    Truncated.typ := CISLO_CELE;
    if typ = CISLO_DESETINNE then Truncated.cele := trunc (desetinne)
                             else Truncated.cele := cele;
end;


function  ODTCislo.Rounded: ODTCislo;
begin
    Rounded := ODTCislo.Create;
    Rounded.typ := CISLO_CELE;
    if typ = CISLO_DESETINNE then Rounded.cele := trunc (desetinne + 0.5)
                             else Rounded.cele := cele;
end;


procedure ODTCislo.DivideBy (const x: ODTCislo);
begin
  try
    if (typ = CISLO_CELE) and (x.typ = CISLO_CELE) then begin
        cele := cele div x.cele;
    end else if typ = CISLO_CELE then begin
        typ := CISLO_DESETINNE;
        desetinne := cele / x.desetinne;
    end else if x.typ = CISLO_CELE then begin
        desetinne := desetinne / x.cele;
    end else
        desetinne := desetinne / x.desetinne;
  except
    raise ODTCisloException.Create(DIV_BY_ZERO);
  end;
end;


function  ODTCislo.getReal: real;
begin
    if typ = CISLO_DESETINNE then getReal := desetinne
                             else getReal := cele;
end;


function  ODTCislo.getInteger: longint;
begin
    if typ = CISLO_CELE then getInteger := cele
                        else getInteger := trunc (desetinne);
end;


function ODTCislo.fSin: ODTCislo;
begin
    fSin := ODTCislo.Create;
    fSin.typ := CISLO_DESETINNE;
    if typ = CISLO_DESETINNE then fSin.desetinne := sin (desetinne)
                             else fSin.desetinne := sin (cele);
end;


function ODTCislo.fCos: ODTCislo;
begin
    fCos := ODTCislo.Create;
    fCos.typ := CISLO_DESETINNE;
    if typ = CISLO_DESETINNE then fCos.desetinne := cos (desetinne)
                             else fCos.desetinne := cos (cele);
end;


function ODTCislo.fLn: ODTCislo;
begin
  try
    fLn := ODTCislo.Create;
    fLn.typ := CISLO_DESETINNE;
    if typ = CISLO_DESETINNE then fLn.desetinne := ln (desetinne)
                             else fLn.desetinne := ln (cele);
  except
      raise ODTCisloException.Create(INVALID_LOG_ARGUMENT);
  end;
end;


function ODTCislo.fLog: ODTCislo;
begin
  try
    fLog := ODTCislo.Create;
    fLog.typ := CISLO_DESETINNE;
    if typ = CISLO_DESETINNE then fLog.desetinne := ln (desetinne) / ln (10)
                             else fLog.desetinne := ln (cele) / ln (10);
  except
      raise ODTCisloException.Create(INVALID_LOG_ARGUMENT);
  end;
end;


function ODTCislo.fSqrt: ODTCislo;
begin
  try
    fSqrt := ODTCislo.Create;
    fSqrt.typ := CISLO_DESETINNE;
    if typ = CISLO_DESETINNE then fSqrt.desetinne := sqrt (desetinne)
                             else fSqrt.desetinne := sqrt (cele);
  except
      raise ODTCisloException.Create(INVALID_SQRT_ARGUMENT);
  end;
end;


end.
