unit UnitPromenne;

interface


uses UnitEditaceProcedury, Contnrs, Classes, UnitObecnyDatovyTyp, UnitDefiniceVyjimek, Dialogs, Sysutils, IniFiles;

const MAXLEVEL = 500;
{max. vyska zasobniku promennych, kazda uroven odpovida jednomu
 "scopu" lokalnich promennych pouzitych v programu}

type
  Vars = class (TObject)
  {slouzi k implementaci mechanismu lokalnich promennych}
    private
      zasobnik: array[1..MAXLEVEL] of TStringList;
      vyskaZas: integer;
    public
      constructor Create;
      destructor Destroy; override;
      procedure zvysVysku; {nove definovane promenne prekryji ty stare}
      procedure snizVysku; {zahod promenne definovane od posledniho volani zvysVysku}
      procedure pridejPromennou (const nazev: String; const hodnota: ODT);
      function  najdiPromennou (const nazev: String): ODT;
  end;

implementation


destructor Vars.Destroy;
{vyprazdni zasobnik a uvolni pamet}
begin
    while vyskaZas > 0 do snizVysku;
    inherited Destroy;
end;


constructor Vars.Create;
begin
    inherited Create;
    vyskaZas := 0;
end;


procedure Vars.zvysVysku;
{typicky se vola pri zavolani nove procedury, zvyseni zasobniku
 zpusobi, ze lze nove definovat i procedury, ktere maji stejny
 nazev jako procedury definovane drive, hodnoty starych procedur
  budou prekryty novymi}
begin
    if vyskaZas >= MAXLEVEL then
        raise RuntimeException.Create (VARIABLE_STACK_OVERFLOW);
    inc (vyskaZas);
    zasobnik[vyskaZas] := TStringList.Create;
    with zasobnik[vyskaZas] do begin
        Sorted := true;
        CaseSensitive := true;
        Duplicates := dupError;
    end;
end;


procedure Vars.snizVysku;
begin
    zasobnik[vyskaZas].Clear;
    zasobnik[vyskaZas].Destroy;
    dec (vyskaZas);
end;


procedure Vars.pridejPromennou (const nazev: String; const hodnota: ODT);
{prida novou promennou do nejvyssi urovne zasobniku}
begin
    try
        zasobnik[vyskaZas].AddObject(nazev, hodnota);
    except
        raise VarsException.Create(VAR_ALREADY_EXISTS);
    end;
end;


function Vars.najdiPromennou (const nazev: String): ODT;
{Najde promennou -- nejprve se snazi hledat v nejvyssi urovni zasobniku,
 pokud ji nenajde, prejde o stupen nize, a tak dale, dokud
 nenarazi na dno zasobniku. Pak ohlasi, ze promennou nemuze najit.}
var i, index: integer;
begin
    for i := vyskaZas downto 1 do
        if zasobnik[i].Find(nazev, index) then begin
            najdiPromennou := ODT (zasobnik[i].Objects[index]);
            exit;
        end;
    raise VarsException.Create(VAR_NOT_FOUND);
end;


end.
 