unit UnitZasobnikyParametruAPozic;

interface

uses UnitProcedura, UnitObecnyDatovyTyp, UnitDefiniceVyjimek, Dialogs, SysUtils;

const MAX_PARAM_STACK_DEPTH = 2000;
const MAX_CALL_STACK_DEPTH = 500;


type

Pozice = record
    {rika, v ktere procedure a na jake pozici se prave nachazime}
    proc: Procedura;
    znak: integer;
end;

Zasobniky = class
  private
    paramStack: record
       {zasobnik parametru predavanych pri volani procedury}
       data:     array[1..MAX_PARAM_STACK_DEPTH] of ODT;
       vyskaZas: integer;
    end; {of record paramStack}
    callStack: record
       {zasobnik, na nejz se ukladaji pozice, kam je treba se vratit po vykonani zavolane procedury}
       data: array[1..MAX_CALL_STACK_DEPTH] of Pozice;
       vyskaZas: integer;
    end; {of record callStack}
  public
    constructor Create;
    procedure newCall (p: Procedura; znak: integer);
    procedure newParam (parametr: ODT);
    function noCalls: boolean;
    function noParams: boolean;
    function popCall: Pozice;
    function popParam: ODT;
end;


implementation


constructor Zasobniky.Create;
begin
    inherited Create;
    paramStack.vyskaZas := 0;
    callStack.vyskaZas := 0;
end;


procedure Zasobniky.newCall (p: Procedura; znak: integer);
{prida novou pozici do zasobniku}
begin
    with callStack do begin
        if vyskaZas >= MAX_CALL_STACK_DEPTH then
            raise RuntimeException.Create (CALL_STACK_OVERFLOW);
        inc (vyskaZas);
        data[vyskaZas].proc := p;
        data[vyskaZas].znak := znak;
    end;
end;


procedure Zasobniky.newParam (parametr: ODT);
{prida novy parametr do zasobniku}
begin
    with paramStack do begin
        if vyskaZas >= MAX_PARAM_STACK_DEPTH then
            raise RuntimeException.Create (PARAM_STACK_OVERFLOW);
        inc (vyskaZas);
        data[vyskaZas] := parametr;
    end;
end;


function Zasobniky.noCalls: boolean;
{true pokud na zasobniku nejsou zadna volani procedur}
begin
    noCalls := (callStack.vyskaZas = 0);
end;


function Zasobniky.noParams: boolean;
{true pokud na zasobniku nejsou zadne parametry}
begin
    noParams := (paramStack.vyskaZas = 0);
end;


function Zasobniky.popCall: Pozice;
{pop provedena na zasobniku pozic}
begin
    with callStack do begin
        popCall := data[vyskaZas];
        dec (vyskaZas);
    end;
end;


function Zasobniky.popParam: ODT;
{pop provedena na zasobniku pozic}
begin
    with paramStack do begin
        popParam := data[vyskaZas];
        dec (vyskaZas);
    end;
end;

end.
 