unit ZImage;
{komponenta -- plocha, na niz zelvicka kresli}

interface



uses
  SysUtils, Classes, Controls, ExtCtrls, Graphics, Types, Contnrs,
  UnitDefiniceVyjimek;

type

  Stav = record
        x, y: real;   {aktualni poloha}
        fi: real;     {smer, jimz je zelvicka natocena, v radianech, 0 znamena doprava}
        peroDole: boolean;   {je-li true, zelvicka pri pohybu zanechava stopu}
        kreslitZelvicku: boolean;   {zelvicka -- modra sipka -- je zobrazena}
  end;

  TZImage = class(TImage)
  private

  protected
    aktStav: Stav;  {= aktualni stav}
    BM: TBitmap;    {obrazek, na nemz je zachycen stav plochy vyjma zelvicky}
    bPozadi, bZelvicky: TColor; {barvy pozadi a zelvy}
    function colorNameToCode (s: String): TColor; {prevadi napr. cervena -> clRed}
  public
    procedure init;
    procedure vymazat;
    procedure kresli;
    procedure kresliZelvicku;
    procedure nekresliZelvicku;
    procedure zvedniPero;
    procedure polozPero;
    procedure sirkaPera (w: integer);
    procedure barvaPera (c: String);
    procedure barvaZelvicky (c: String);
    procedure vpred (d: real);
    procedure otoc (x: real);
  published
    { Published declarations }
  end;

procedure Register;

implementation


procedure TZImage.sirkaPera (w: integer);
begin
    BM.Canvas.Pen.Width := w;
end;


function TZImage.colorNameToCode (s: String): TColor;
{prevadi napr. cervena -> clRed}
begin
         if s = 'cervena'    then colorNameToCode := clRed
    else if s = 'modra'      then colorNameToCode := clNavy
    else if s = 'zelena'     then colorNameToCode := clGreen
    else if s = 'zluta'      then colorNameToCode := clYellow
    else if s = 'bila'       then colorNameToCode := clWhite
    else if s = 'cerna'      then colorNameToCode := clBlack
    else if s = 'fialova'    then colorNameToCode := clPurple
    else if s = 'seda'       then colorNameToCode := clGray
    else if s = 'podzimni'   then colorNameToCode := $000BBFF
    else if s = 'oranzova'   then colorNameToCode := $00099FF
    else if s = 'ruzova'     then colorNameToCode := $0AAAAFF
    else if s = 'bledemodra' then colorNameToCode := $0FFCCCC
    else if s = 'hneda'      then colorNameToCode := $0004488
    else raise ParseException.Create (UNKNOWN_COLOR);
end;


procedure TZImage.barvaPera (c: String);
begin
    BM.Canvas.Pen.Color := colorNameToCode (c);
end;


procedure TZImage.barvaZelvicky (c: String);
begin
    bZelvicky := colorNameToCode (c);
end;


procedure TZImage.init;
{inicializace}
begin
    BM := TBitmap.Create;
    BM.Height := Height;
    BM.Width := Width;
    bPozadi := clWhite;
    bZelvicky := clNavy;
    vymazat;
end;


procedure TZImage.vymazat;
{nakresli bile vybarveny obdelnik pres celou plochu -- smaze plochu,
navic obnovi pocatecni stav zelvicky}
const INITPOZICE = 50;
begin
    with aktStav do begin
        x := Width / 2;
        y := Height - INITPOZICE;
        fi := -PI / 2;
        peroDole := true;
        kreslitZelvicku := true;
    end;
    BM.Canvas.Brush.Color := bPozadi;
    BM.Canvas.Pen.Width := 1;
    BM.Canvas.Pen.Color := clBlack;
    BM.Canvas.Rectangle (0, 0, Width, Height);
    kresli;
end;


procedure TZImage.kresli;
{nakresli aktualni stav plochy, pak
dokresli zelvicku (je-li kresleni zelvicky zapnuto)}
var rohy: array[1..3] of TPoint;
const ROZPETI = PI / 5;
const RAMENO = 20;
begin
    Canvas.Draw (0, 0, BM);
    if aktStav.kreslitZelvicku then begin
        rohy[1].X := round (aktStav.x);
        rohy[1].Y := round (aktStav.y);
        rohy[2].X := round (RAMENO*cos (aktStav.fi + ROZPETI / 2 + PI) + aktStav.x);
        rohy[2].Y := round (RAMENO*sin (aktStav.fi + ROZPETI / 2 + PI) + aktStav.y);
        rohy[3].X := round (RAMENO*cos (aktStav.fi - ROZPETI / 2 + PI) + aktStav.x);
        rohy[3].Y := round (RAMENO*sin (aktStav.fi - ROZPETI / 2 + PI) + aktStav.y);
        Canvas.Brush.Color := bZelvicky;
        Canvas.Polygon (rohy);
    end;
    Repaint;
end;


procedure TZImage.kresliZelvicku;
begin
    aktStav.kreslitZelvicku := true;
    kresli;
end;


procedure TZImage.nekresliZelvicku;
begin
    aktStav.kreslitZelvicku := false;
    kresli;
end;


procedure TZImage.zvedniPero;
begin
    aktStav.peroDole := false;
end;


procedure TZImage.polozPero;
begin
    aktStav.peroDole := true;
end;


procedure TZImage.vpred (d: real);
{do BM -- aktualniho stavu plochy -- prikresli usecku delky d aktualnim smerem}
begin
    BM.Canvas.MoveTo (round (aktStav.x), round (aktStav.y));
    aktStav.x := aktStav.x + d*cos (aktStav.fi);
    aktStav.y := aktStav.y + d*sin (aktStav.fi);
    if aktStav.peroDole then
        BM.Canvas.LineTo (round (aktStav.x), round (aktStav.y));
    kresli;
end;


procedure TZImage.otoc (x: real);
{zmeni natoceni zelvy}
begin
    aktStav.fi := aktStav.fi + x;
    kresli;
end;


procedure Register;
{systemova, nesahat!}
begin
  RegisterComponents('Samples', [TZImage]);
end;

end.
